#include "mex.h"
#include "classify.h"

#include <string>
#include <vector>

#include <opencv/cv.h>

using namespace std;
using namespace cv;
using namespace wtf;

#ifndef INDEX
#define INDEX(i,j,m)    ((i)+(j)*(m))
#endif

float classify_single_column_in_matrix (
        float * v,
        float * v_rev,
        int col,
        int dimension,
        CvBoost * model )
{
    size_t offset = INDEX(0,col,dimension);
    float* v_ptr = v+offset;
    Mat v_mat = Mat(1,dimension,CV_32F,v_ptr);
    
    float* v_rev_ptr = v_rev+offset;
    Mat v_rev_mat = Mat(1,dimension,CV_32F,v_rev_ptr);
    
    pair<Mat,Mat> v_pair = pair<Mat,Mat>(v_mat,v_rev_mat);
    
    return classify(v_pair,model);
}
void mexFunction (
        int nlhs, mxArray* plhs[],      /*Output variables */
        int nrhs, const mxArray* prhs[] /*Input variables */
        )
{
    if (nlhs > 1 || nrhs != 3)
    {
        mexErrMsgTxt("Format: scores = classifyMultiplemx(fecture_vectors,feature_vectors_reverse,model_path)");
        return;
    }
    
    float   *v, *v_rev;     /* pointers to feature vectors matrices */
    int     m, n;   /* dimension of the feature vectos and number of them */
    string  modelPath;
    
    float   *dv;    /* decision values for each feature vector */
    
    /* Inputs */
    v       = (float*)mxGetPr(prhs[0]);
    v_rev   = (float*)mxGetPr(prhs[1]);
    m       = mxGetM(prhs[0]);  // dimenstion of feature space
    n       = mxGetN(prhs[0]);  // number of feature vectors
    modelPath = mxArrayToString(prhs[2]);

//     mexPrintf("m = %d\n",m);
    
    /* Output */
    plhs[0] = mxCreateNumericMatrix(n,1,mxSINGLE_CLASS,mxREAL);
    dv      = (float*)mxGetData(plhs[0]);
    
    CvBoost * model = new CvBoost();
    model->load(modelPath.c_str());
//     cout << modelPath << ": "<< model->get_params().weak_count<<endl;
    
    for (int j=0; j<n; j++) {
        float p = classify_single_column_in_matrix(v,v_rev,j,m,model);
        dv[j] = p;
    }
    
    return;
}
