#include "mex.h"
#include "train.h"
#include "def.h"

#include <string>
#include <vector>

#include <opencv/cv.h>

using namespace cv;
using namespace std;
using namespace wtf;

void mexFunction (
    int nlhs, mxArray* plhs[],      /*Output variables */
    int nrhs, const mxArray* prhs[] /*Input variables */
    )
{
    if (nlhs!=0)
    {
        mexErrMsgTxt("Too many outputs. (should be: 0)");
        return;
    }
    if (nrhs > 4)
    {
        mexErrMsgTxt("Format: trainmx(feature_vectors, labels, model_path, [n_iteration])");
        return;
    }    
    
    float   *v;     /* pointers to feature vectors matrix */
    int     *l;     /* pointers to labels vector */
    int     m, n;   /* dimension of the feature vectos and number of them */
    char    *modelPath;    /* number of weak classifiers of AdaBoost */
    int     n_itr;

    v   = (float*)mxGetPr(prhs[0]);
    l   = (int*)mxGetPr(prhs[1]);
    m   = mxGetM(prhs[0]);  // dimension of feature space
    n   = mxGetN(prhs[0]);  // number of feature vectors
    modelPath = mxArrayToString(prhs[2]);
    
    if (n!=mxGetN(prhs[1])&&n!=mxGetM(prhs[1])) {
        mexErrMsgTxt("Number of feature vectors and labels mismatch!");
    }
    
    mexPrintf("Training on %d samples of %d-dimension...\n",n,m);
    
    CvBoost *model;
    if (nrhs > 3) {
        n_itr = (int)mxGetScalar(prhs[3]);
    } else {
        n_itr = DEFAULT_ITR;    // defined in def.h
    }
    
    mexPrintf("AdaBoost with %d weak classifiers\n",n_itr);
    model = train(v,l,m,n,n_itr);
    mexPrintf("Saving the trained model to %s\n",modelPath);
    model->save(modelPath);
    
    return;
}
