#include "classify.h"

using namespace std;
using namespace cv;

namespace wtf {

/*******************************
 * Public functions in libwtf.h
 *******************************/
float classify (
    const pair<std::vector<float>,std::vector<float> > &v,
    const CvBoost * const model )
{
    Mat m1(v.first);
    m1 = m1.reshape(1,1);   // Do a O(1) transpose; avoid using m1.t() which invloves copying data. 
    Mat m2(v.second);
    m2 = m2.reshape(1,1);
    
    bool returnSum = true;	// If true then predict() returns sum of votes instead of the class label. A negative value “votes” for class #0, a positive value - for class #1.
    // http://docs.opencv.org/modules/ml/doc/boosting.html#cvboosttree
    // http://docs.opencv.org/modules/ml/doc/boosting.html#cvboost-predict

    float p_first = model
            ->predict(m1,Mat(),Range::all(),false,returnSum);
    float p_second = model
            ->predict(m2,Mat(),Range::all(),false,returnSum);    
    return min(p_first,p_second);
}
/* END of public functions in libwtf.h 
 **************************************/


/**************************
 * Functions in classify.h 
 **************************/
float classify (
    const pair<Mat,Mat> &v,
    const CvBoost * const model )
{    
    float p_first = model
            ->predict(v.first,Mat(),Range::all(),false,true);
    float p_second = model
            ->predict(v.second,Mat(),Range::all(),false,true);    
    return min(p_first,p_second);
}
/* END of functions in classify.h 
 *********************************/

}  // namespace wtf